package com.huiju.estateagents.third.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huiju.estateagents.base.ResponseBean;
import com.huiju.estateagents.common.WxMpUtils;
import com.huiju.estateagents.redpack.entity.TaMpPerson;
import com.huiju.estateagents.third.entity.TaThirdPartyMiniappConfig;
import com.huiju.estateagents.third.service.ITaThirdPartyMiniappConfigService;
import lombok.AllArgsConstructor;
import me.chanjar.weixin.common.error.WxErrorException;
import me.chanjar.weixin.mp.api.WxMpService;
import me.chanjar.weixin.mp.bean.result.WxMpOAuth2AccessToken;
import me.chanjar.weixin.mp.bean.result.WxMpUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;

/**
 * @author Edward
 */
@AllArgsConstructor
@Controller
@RequestMapping("/api")
public class WxRedpackController {
    public WxMpUtils wxMpUtils;

    @Autowired
    public ITaThirdPartyMiniappConfigService iTaThirdPartyMiniappConfigService;

    /**
     * 换取openid接口并插入公众号人员表
     * @param mpAppid
     * @param code
     * @param request
     * @return
     */
    @GetMapping("/third/redirect/{mpAppid}/greet")
    public ResponseBean greetUser(@PathVariable String mpAppid, @RequestParam String code, @RequestParam String personId, HttpServletRequest request) {
        WxMpService wxService = wxMpUtils.wxMpService();
        if (!wxService.switchover(mpAppid)) {
            throw new IllegalArgumentException(String.format("未找到对应appid=[%s]的配置，请核实！", mpAppid));
        }
        TaMpPerson taMpPerson = new TaMpPerson();
        try {
            //换取openid
            WxMpOAuth2AccessToken accessToken = wxService.oauth2getAccessToken(code);
            WxMpUser user = wxService.oauth2getUserInfo(accessToken, null);

            //获取H5项目相关信息
            String xAppid = request.getHeader("x-appid");
             //获取小程序appid
            String miniAppid = request.getHeader("appid");
            //插入公众号人员信息并插入表
            taMpPerson = iTaThirdPartyMiniappConfigService.newMpPerson(user, mpAppid, xAppid, miniAppid, personId);
        } catch (WxErrorException e) {
            e.printStackTrace();
        }
        return ResponseBean.success(taMpPerson);
    }

    /**
     * 我要参加 活动接口
     * @param mpAppid
     * @param mpPersonId
     * @param request
     * @return
     */
    @PostMapping("/third/activity/{mpAppid}/participate")
    public ResponseBean participateActivity(@PathVariable String mpAppid, @RequestParam String mpPersonId, HttpServletRequest request) {
        //获取H5项目相关信息
        String xAppid = request.getHeader("x-appid");
        QueryWrapper<TaThirdPartyMiniappConfig> taThirdPartyMiniappConfigQueryWrapper = new QueryWrapper<>();

        taThirdPartyMiniappConfigQueryWrapper.eq("appid", xAppid);
        TaThirdPartyMiniappConfig taThirdPartyMiniappConfig = iTaThirdPartyMiniappConfigService.getOne(taThirdPartyMiniappConfigQueryWrapper);
        if (null == taThirdPartyMiniappConfig){
            return ResponseBean.error("不存在的H5活动",ResponseBean.ERROR_UNAVAILABLE);
        }
        try {
            ResponseBean res = iTaThirdPartyMiniappConfigService.participateActivity(mpAppid,mpPersonId,taThirdPartyMiniappConfig.getTargetId(),taThirdPartyMiniappConfig.getOrgId());
            return res;
        }catch (Exception e){
            e.printStackTrace();
            return ResponseBean.error("程序出错请联系运营人员",ResponseBean.ERROR_ILLEGAL_PARAMS);
        }
    }

    /**
     * 帮他助力 接口
     * @param mpAppid
     * @param mpPersonId
     * @param request
     * @return
     */
    @PostMapping("/third/activity/{mpAppid}/help")
    public ResponseBean helpActivity(@PathVariable String mpAppid, @RequestParam String mpPersonId, @RequestParam String parentMpPersonId ,HttpServletRequest request) {
        //获取H5项目相关信息
        String xAppid = request.getHeader("x-appid");
        QueryWrapper<TaThirdPartyMiniappConfig> taThirdPartyMiniappConfigQueryWrapper = new QueryWrapper<>();

        taThirdPartyMiniappConfigQueryWrapper.eq("appid", xAppid);
        TaThirdPartyMiniappConfig taThirdPartyMiniappConfig = iTaThirdPartyMiniappConfigService.getOne(taThirdPartyMiniappConfigQueryWrapper);
        if (null == taThirdPartyMiniappConfig){
            return ResponseBean.error("不存在的H5活动",ResponseBean.ERROR_UNAVAILABLE);
        }
        try {
            ResponseBean res = iTaThirdPartyMiniappConfigService.helpActivity(mpAppid, mpPersonId, parentMpPersonId, taThirdPartyMiniappConfig.getTargetId(),taThirdPartyMiniappConfig.getOrgId());
            return res;
        }catch (Exception e){
            e.printStackTrace();
            return ResponseBean.error("程序出错请联系运营人员",ResponseBean.ERROR_ILLEGAL_PARAMS);
        }
    }
}
