package com.huiju.estateagents.third.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huiju.estateagents.base.ResponseBean;
import com.huiju.estateagents.common.CommConstant;
import com.huiju.estateagents.common.MD5Utils;
import com.huiju.estateagents.drainage.entity.TaDrainage;
import com.huiju.estateagents.drainage.mapper.TaDrainageMapper;
import com.huiju.estateagents.redpack.entity.*;
import com.huiju.estateagents.redpack.mapper.*;
import com.huiju.estateagents.third.entity.TaThirdPartyMiniappConfig;
import com.huiju.estateagents.third.mapper.TaThirdPartyMiniappConfigMapper;
import com.huiju.estateagents.third.service.ITaThirdPartyMiniappConfigService;
import me.chanjar.weixin.mp.bean.result.WxMpUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 *   服务实现类
 * </p>
 *
 * @author jobob
 * @since 2019-10-17
 */
@Service
public class TaThirdPartyMiniappConfigServiceImpl extends ServiceImpl<TaThirdPartyMiniappConfigMapper, TaThirdPartyMiniappConfig> implements ITaThirdPartyMiniappConfigService {
	
	@Autowired
	private TaThirdPartyMiniappConfigMapper taThirdPartyMiniappConfigMapper;

	@Autowired
	private TaMpPersonMapper taMpPersonMapper;

	@Autowired
	private TaMpMiniappMapper taMpMiniappMapper;

	@Autowired
	private TaDrainageMapper taDrainageMapper;

	@Autowired
	private TaRedPacketMapper taRedPacketMapper;

	@Autowired
	private TaRedPacketPersonMapper taRedPacketPersonMapper;

	@Autowired
	private TaRedPacketHelpMapper taRedPacketHelpMapper;

	@Autowired
	private TaRedPacketHistoryMapper taRedPacketHistoryMapper;

	@Autowired
	private TaRedPacketMoneyMapper taRedPacketmonetMapper;

	/**
	 * 校验是否合法第三方接口调用
	 *
	 * @param tokenHeader
	 * @param appidHeader
	 * @param nowHeader
	 * @return
	 */
	@Override
	public String checkToken(String tokenHeader, String appidHeader, String nowHeader) {
		//秘钥是否过期
		QueryWrapper<TaThirdPartyMiniappConfig> queryWrapper = new QueryWrapper<>();
		queryWrapper.eq("appid",appidHeader);
		TaThirdPartyMiniappConfig taThirdPartyMiniappConfig = taThirdPartyMiniappConfigMapper.selectOne(queryWrapper);
//		if (LocalDateTime.now().isAfter(taThirdPartyMiniappConfig.getExpirationDate())){
//			return "秘钥已过期，请重新申请";
//		}
		
		//判断nowHeader是十五分钟之内(15 * 60 * 1000)
		if (System.currentTimeMillis() - Long.valueOf(nowHeader) > (15 * 60 * 1000)){
			return "密钥超时，请在十五分钟内请求";
		}
		
		//重新加密对比
		StringBuilder stringBuilder = new StringBuilder();
		stringBuilder.append(taThirdPartyMiniappConfig.getAppid()).append(taThirdPartyMiniappConfig.getAppSecret()).append(nowHeader);
		String token = MD5Utils.md5(MD5Utils.md5(stringBuilder.toString()));
		if (!tokenHeader.equals(token)){
			return "秘钥出错请重新生成";
		}
		
		return "success";
	}

	/**
	 * 创建公众号人员并关联上小程序人员
	 *
	 * @param user
	 * @param xAppid
	 * @param miniAppid
	 * @param personId
	 */
	@Override
	public TaMpPerson newMpPerson(WxMpUser user,String mpAppid, String xAppid, String miniAppid, String personId) {
		//查询这个人员是否存在
		TaMpPerson mpPerson = taMpPersonMapper.selectOne(new QueryWrapper<TaMpPerson>().eq("mp_openid", user.getOpenId()));
		//不存在就新建
		if (null == mpPerson){
			//新增入公众号人员表
			TaMpPerson taMpPerson = new TaMpPerson();
			taMpPerson.setAppid(mpAppid);
			taMpPerson.setCity(user.getCity());
			taMpPerson.setCountry(user.getCountry());
			taMpPerson.setCreateDate(LocalDateTime.now());
			taMpPerson.setHeadimgurl(user.getHeadImgUrl());
			taMpPerson.setMpOpenid(user.getOpenId());
			taMpPerson.setMpPersonId(personId);
			taMpPerson.setNickname(user.getNickname());
			taMpPerson.setProvince(user.getProvince());
			taMpPerson.setSex(user.getSex());
			taMpPerson.setStatus(CommConstant.STATUS_NORMAL);
			taMpPersonMapper.insert(taMpPerson);
			mpPerson = taMpPerson;
		}

		//查看这个人是否绑定了小程序用户
		TaMpMiniapp taMpMiniapp = taMpMiniappMapper.selectOne(new QueryWrapper<TaMpMiniapp>().eq("mp_person_id", mpPerson.getMpPersonId()).eq("miniapp_person_id", personId));
		if (null == taMpMiniapp){
			taMpMiniapp = new TaMpMiniapp();
			taMpMiniapp.setMpPersonId(mpPerson.getMpPersonId());
			taMpMiniapp.setMiniappPersonId(personId);
			taMpMiniappMapper.insert(taMpMiniapp);
		}

		return mpPerson;
	}

	/**
	 * 参加活动生成参与记录等
	 *
	 * @param mpAppid 公众号appid
	 * @param mpPersonId 公众号人员id
	 * @param drainageId H5活动id
	 * @param orgId 小程序id
	 */
	@Override
	public ResponseBean participateActivity(String mpAppid, String mpPersonId, String drainageId, Integer orgId) {
		//第一步根据drainggeid查询出红包表对应的记录
		TaDrainage taDrainage = taDrainageMapper.selectById(drainageId);
		//第二步查询出这个H5对应的活动
		TaRedPacket taRedPacket = taRedPacketMapper.selectById(taDrainage.getActivityId());
		//判断这个活动的过期时间
		if (LocalDateTime.now().isAfter(taRedPacket.getEndDate())){
			return ResponseBean.error(taRedPacket.getActivityEndTip(),ResponseBean.ERROR_ILLEGAL_PARAMS);
		}
		if (LocalDateTime.now().isBefore(taRedPacket.getStartDate())){
			return ResponseBean.error("此活动未开始",ResponseBean.ERROR_ILLEGAL_PARAMS);
		}

		//参与活动
		TaRedPacketPerson taRedPacketPerson = saveRedPacketPerson(mpPersonId,taRedPacket,orgId);

		return ResponseBean.success(taRedPacketPerson);
	}

	/**
	 * 参与活动方法--悲观锁
	 * @param mpPersonId
	 * @param taRedPacket
	 * @param orgId
	 */
	private synchronized TaRedPacketPerson saveRedPacketPerson(String mpPersonId, TaRedPacket taRedPacket, Integer orgId) {
		//查看这个人绑定的小程序用户
		TaMpMiniapp taMpMiniapp = taMpMiniappMapper.selectOne(new QueryWrapper<TaMpMiniapp>().eq("mp_person_id", mpPersonId));

		//查看此人是否参加过活动如果参加过直接返回
		QueryWrapper<TaRedPacketPerson> queryWrapper = new QueryWrapper<>();
		queryWrapper.eq("activity_id",taRedPacket.getActivityId());
		queryWrapper.eq("org_id",orgId);
		queryWrapper.eq("mp_person_id",mpPersonId);
		queryWrapper.eq("miniapp_person_id",taMpMiniapp.getMiniappPersonId());
		TaRedPacketPerson redPacketPerson = taRedPacketPersonMapper.selectOne(queryWrapper);
		if (null != redPacketPerson){
			return redPacketPerson;
		}

		//参与活动
		TaRedPacketPerson taRedPacketPerson = new TaRedPacketPerson();
		taRedPacketPerson.setActivityId(taRedPacket.getActivityId());
		taRedPacketPerson.setCreateDate(LocalDateTime.now());
		taRedPacketPerson.setIsSignedUp(true);
		taRedPacketPerson.setMiniappPersonId(taMpMiniapp.getMiniappPersonId());
		taRedPacketPerson.setMpPersonId(mpPersonId);
		taRedPacketPerson.setOrgId(orgId);
		taRedPacketPersonMapper.insert(taRedPacketPerson);
		return taRedPacketPerson;
	}

	/**
	 * 助力活动接口
	 *
	 * @param mpAppid 公众号appid
	 * @param mpPersonId 公众号personId
	 * @param parentMpPersonId 公众号被助力者的peisonId
	 * @param drainageId H5活动id
	 * @param orgId 小程序id
	 * @return
	 */
	@Override
	public ResponseBean helpActivity(String mpAppid, String mpPersonId, String parentMpPersonId, String drainageId, Integer orgId) {
		//第一步根据drainggeid查询出红包表对应的记录
		TaDrainage taDrainage = taDrainageMapper.selectById(drainageId);
		//第二步查询出这个H5对应的活动
		TaRedPacket taRedPacket = taRedPacketMapper.selectById(taDrainage.getActivityId());
		//判断这个活动的过期时间
		if (LocalDateTime.now().isAfter(taRedPacket.getEndDate())){
			return ResponseBean.error(taRedPacket.getActivityEndTip(),ResponseBean.ERROR_ILLEGAL_PARAMS);
		}
		if (LocalDateTime.now().isBefore(taRedPacket.getStartDate())){
			return ResponseBean.error("此活动未开始",ResponseBean.ERROR_ILLEGAL_PARAMS);
		}

		//查看这个人绑定的小程序用户
		TaMpMiniapp taMpMiniapp = taMpMiniappMapper.selectOne(new QueryWrapper<TaMpMiniapp>().eq("mp_person_id", mpPersonId));

		//查询被助力者的绑定的小程序用户
		TaMpMiniapp parentMpMiniapp = taMpMiniappMapper.selectOne(new QueryWrapper<TaMpMiniapp>().eq("mp_person_id", parentMpPersonId));

		//根据活动id查询这个活动的助力规则
		TaRedPacketHelp taRedPacketHelp = taRedPacketHelpMapper.selectById(taDrainage.getActivityId());
		
		//助力
		String result = saveRedPacketHistory(taMpMiniapp, parentMpMiniapp, taRedPacketHelp, orgId);
		if (result.equals("success")){
			ResponseBean.error(result,ResponseBean.ERROR_ILLEGAL_PARAMS);
		}

		return ResponseBean.success(taRedPacketHelp.getSuccessTip());
	}

	private synchronized String saveRedPacketHistory(TaMpMiniapp taMpMiniapp, TaMpMiniapp parentMpMiniapp, TaRedPacketHelp taRedPacketHelp, Integer orgId) {
		//查看我是否助力过
		QueryWrapper<TaRedPacketHistory> queryWrapper = new QueryWrapper();
		queryWrapper.eq("activity_id",taRedPacketHelp.getActivityId());
		queryWrapper.eq("org_id",orgId);
		queryWrapper.eq("from_mp_person",taMpMiniapp.getMpPersonId());
		queryWrapper.eq("from_miniapp_person",taMpMiniapp.getMiniappPersonId());
		List<TaRedPacketHistory> taRedPacketHistories = taRedPacketHistoryMapper.selectList(queryWrapper);
		//校验总助力次数
		if (taRedPacketHistories.size() >= taRedPacketHelp.getLimitNumPerPerson()){
			return taRedPacketHelp.getLimitPerPersonIllegalTip();
		}
		//校验单人每日助力次数限制
		List<TaRedPacketHistory> dayHelpList = taRedPacketHistories.stream().filter(e -> e.getCreateDate().toLocalDate().equals(LocalDateTime.now().toLocalDate())).collect(Collectors.toList());
		if (dayHelpList.size() >= taRedPacketHelp.getLimitNumPerDay()){
			return taRedPacketHelp.getLimitPerDayIllegalTip();
		}
		//校验为同一人助力次数限制
		List<TaRedPacketHistory> eachHelpList = taRedPacketHistories.stream().filter(e -> e.getTargetMiniappPerson().equals(parentMpMiniapp.getMiniappPersonId())).collect(Collectors.toList());
		if (eachHelpList.size() >= taRedPacketHelp.getLimitNumForEachPerson()){
			return taRedPacketHelp.getLimitForEachPersonIllegalTip();
		}

		//校验还有此次助力成功后是否需要发送红包
		QueryWrapper<TaRedPacketHistory> parentRedPacketHistory = new QueryWrapper();
		parentRedPacketHistory.eq("activity_id",taRedPacketHelp.getActivityId());
		parentRedPacketHistory.eq("org_id",orgId);
		parentRedPacketHistory.eq("target_mp_person",parentMpMiniapp.getMpPersonId());
		parentRedPacketHistory.eq("target_miniapp_person",parentMpMiniapp.getMiniappPersonId());
		List<TaRedPacketHistory> parentRedPacketHistories = taRedPacketHistoryMapper.selectList(parentRedPacketHistory);
		Boolean redPackageState = parentRedPacketHistories.size() + 1 <= taRedPacketHelp.getHelpSuccesPersons();


		//助力
		TaRedPacketHistory taRedPacketHistory = new TaRedPacketHistory();
		taRedPacketHistory.setActivityId(taRedPacketHelp.getActivityId());
		taRedPacketHistory.setCreateDate(LocalDateTime.now());
		taRedPacketHistory.setFromMiniappPerson(taMpMiniapp.getMiniappPersonId());
		taRedPacketHistory.setFromMpPerson(taMpMiniapp.getMpPersonId());
		taRedPacketHistory.setOrgId(orgId);
		taRedPacketHistory.setTargetMiniappPerson(parentMpMiniapp.getMiniappPersonId());
		taRedPacketHistory.setTargetMpPerson(parentMpMiniapp.getMpPersonId());
		taRedPacketHistoryMapper.insert(taRedPacketHistory);
		if (redPackageState){
			//发送红包
			sendRedPackage(taMpMiniapp, parentMpMiniapp, taRedPacketHelp, orgId);
		}
		return "success";
	}

	/**
	 * 发送红包
	 * @param taMpMiniapp
	 * @param parentMpMiniapp
	 * @param taRedPacketHelp
	 * @param orgId
	 */
	private synchronized void sendRedPackage(TaMpMiniapp taMpMiniapp, TaMpMiniapp parentMpMiniapp, TaRedPacketHelp taRedPacketHelp, Integer orgId) {
		TaRedPacketMoney taRedPacketMoney = taRedPacketmonetMapper.selectById(taRedPacketHelp.getActivityId());
		
	}
}
